/* test_extract.cpp
 * Copyright (C) 2010, Francisco Claude, all rights reserved.
 *
 * Program to test "extract" in the int-based SSA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include <ssa_words.h>
#include <iostream>
#include <fstream>
#include <sstream>

using namespace std;

bool check_extract(uint * text, uint n, uint * snippet, uint m, uint i) {
  for(uint j=0;j<m;j++)
    if(text[i+j]!=snippet[j])
      return false;
  return true;
}


int main(int argc, char ** argv) {
  if(argc!=5) {
    cout << "usage: " << argv[0] << " <int-file> <index-file> <pattern-length> <repetitions>" << endl;
    return 0;
  }

  fstream input(argv[1],ios::in | ios::binary);
  if(!input.is_open()) {
    cout << "Error opening file: " << argv[1] << endl;
    return -1;
  }

  input.seekg(0,ios::end);
  uint n=input.tellg()/4;
  uint * text = new uint[n+1];

  input.seekg(0,ios::beg);
  input.read((char*)text,sizeof(uint)*n);
  input.close();
  //for(uint i=0;i<n;i++) text[i]++;
  text[n] = 0;

  bool err;
  FILE * fp = fopen(argv[2],"r");
  ssa_words * ssa = new ssa_words(fp,err);
  fclose(fp);
  ssa->print_stats();

  uint m;
  {
    stringstream ss;
    ss << argv[3];
    ss >> m;
  }

  uint rep;
  {
    stringstream ss;
    ss << argv[4];
    ss >> rep;
  }

  for(uint i=0;i<rep;i++) {
    uint pos = rand()%(n-m-1);
    uint * snippet = ssa->extract(pos,pos+m-1);
    if(!check_extract(text,n,snippet,m,pos)) {
      cout << "Error for pattern " << i+1 << endl;
      cout << "Length " << ssa->length() << endl;
      cout << "Position: " << pos << " length: " << m << endl;
      cout << "Expected: ";
      for(uint k=0;k<m;k++) cout << "<" << text[pos+k] << ">";
      cout << endl;
      cout << "Extract:  ";
      delete [] snippet;
      uint * snippet = ssa->extract(pos-2,pos+m+1);
      for(uint k=0;k<m+4;k++) cout << "<" << snippet[k] << ">";
      cout << endl;
      delete [] snippet;
      break;
    }
    delete [] snippet;
  }

  cout << "Done" << endl;

  delete ssa;
  delete [] text;

  return 0;
}
